uniform mediump float APERTURE_REAL_SIZE;
uniform mediump float FOCAL;
uniform mediump float FOCUS_DISTANCE;
uniform highp float PIXEL_SIZE;
uniform highp float FAR_PLANE;

uniform lowp sampler2D TEXTURE0; /* depth */
uniform lowp sampler2D TEXTURE1; /* orginalny obraz */
uniform lowp sampler2D TEXTURE2;
uniform lowp sampler2D TEXTURE3;
uniform lowp sampler2D TEXTURE4;
uniform lowp sampler2D TEXTURE5;
uniform lowp sampler2D TEXTURE6;
uniform lowp sampler2D TEXTURE7;

varying mediump vec2 TexCoord;

void main()
{
	/* mediump color dlatego bo sumuje */
	mediump vec3 color;
	
	highp vec4 packedDepth = texture2D(TEXTURE0, TexCoord);
	highp float depth = packedDepth.x + packedDepth.y / 256.0 + packedDepth.z / 65536.0 + packedDepth.w / 16777216.0;
	
	highp float distance = depth * FAR_PLANE;
	
	highp float cof = abs(APERTURE_REAL_SIZE * (FOCAL * (FOCUS_DISTANCE - distance)) / (distance * (FOCUS_DISTANCE - FOCAL)));
	
	mediump float mipmap = min(max(log2(cof / PIXEL_SIZE), 0.0), 6.0);
	
	/*if (mipmap < 1.0)
		color = texture2D(TEXTURE2, TexCoord).rgb * mipmap + texture2D(TEXTURE1, TexCoord).rgb * (1.0 - mipmap);
	else if (mipmap < 2.0)
		color = texture2D(TEXTURE3, TexCoord).rgb * (mipmap - 1.0) + texture2D(TEXTURE2, TexCoord).rgb * (1.0 - (mipmap - 1.0));
	else if (mipmap < 3.0)
		color = texture2D(TEXTURE4, TexCoord).rgb * (mipmap - 2.0) + texture2D(TEXTURE3, TexCoord).rgb * (1.0 - (mipmap - 2.0));
	else if (mipmap < 4.0)
		color = texture2D(TEXTURE5, TexCoord).rgb * (mipmap - 3.0) + texture2D(TEXTURE4, TexCoord).rgb * (1.0 - (mipmap - 3.0));
	else if (mipmap < 5.0)
		color = texture2D(TEXTURE6, TexCoord).rgb * (mipmap - 4.0) + texture2D(TEXTURE5, TexCoord).rgb * (1.0 - (mipmap - 4.0));
	else if (mipmap < 6.0)
		color = texture2D(TEXTURE7, TexCoord).rgb * (mipmap - 5.0) + texture2D(TEXTURE6, TexCoord).rgb * (1.0 - (mipmap - 5.0));
	else
		color = texture2D(TEXTURE7, TexCoord).rgb;*/
	
	mipmap = clamp(mipmap, 0.0, 6.0);
	color = texture2D(TEXTURE1, TexCoord).rgb * clamp(1.0 - mipmap, 0.0, 1.0) +
			texture2D(TEXTURE2, TexCoord).rgb * clamp(1.0 - abs(mipmap - 1.0), 0.0, 1.0) + 
			texture2D(TEXTURE3, TexCoord).rgb * clamp(1.0 - abs(mipmap - 2.0), 0.0, 1.0) + 
			texture2D(TEXTURE4, TexCoord).rgb * clamp(1.0 - abs(mipmap - 3.0), 0.0, 1.0) + 
			texture2D(TEXTURE5, TexCoord).rgb * clamp(1.0 - abs(mipmap - 4.0), 0.0, 1.0) + 
			texture2D(TEXTURE6, TexCoord).rgb * clamp(1.0 - abs(mipmap - 5.0), 0.0, 1.0) + 
			texture2D(TEXTURE7, TexCoord).rgb * clamp(1.0 - abs(mipmap - 6.0), 0.0, 1.0);
	
	gl_FragColor = vec4(color, 1.0);
}
