uniform mediump float FRESNEL;

uniform mediump mat3 EVIRONMENT_TEXTURE_MATRIX;

uniform lowp vec4 MATERIAL_DIFFUSE;
uniform lowp vec3 MATERIAL_ENVIRONMENT;

/* Diffuse and specular */
uniform lowp sampler2D TEXTURE0;

/* Normal map (RGB) */
uniform lowp sampler2D TEXTURE1;

/* Lightmap */
uniform lowp sampler2D TEXTURE2;

/* enviroment map */
uniform lowp samplerCube TEXTURE3;

varying lowp vec3 Normal;
varying lowp vec3 Tangent;
varying lowp vec3 Bitangent;
varying mediump vec2 TexCoord;
varying mediump vec2 TexCoord2;
varying mediump vec3 ViewDirAndDist;

void main()
{	
	mediump vec3 ViewDir = normalize(ViewDirAndDist);

	lowp vec4 materialDiffuseAndSpec = texture2D(TEXTURE0, TexCoord);

	mediump vec3 color = materialDiffuseAndSpec.rgb * texture2D(TEXTURE2, TexCoord2).rgb;
	
	lowp vec4 tex1 = texture2D(TEXTURE1, TexCoord);
	lowp vec3 normalMap = vec3(tex1.r * 2.0 - 1.0, tex1.g * 2.0 - 1.0, tex1.b);
	
	lowp vec3 normal = normalMap.x * Tangent + normalMap.y * Bitangent + normalMap.z * Normal;	
	
	mediump vec3 envDir = -reflect(ViewDir, normal); /* minus bo viewDir to eye-vertex, a reflect to ViewDir - 2*dot(ViewDir, normal) * normal */
	mediump vec3 envTexCoord = EVIRONMENT_TEXTURE_MATRIX * envDir;	
	lowp vec3 enviroment = textureCube(TEXTURE3, envTexCoord).rgb;	
	
	mediump float fres = 1.0 - clamp(dot(normal, ViewDir), 0.0, 1.0); /* musi byc clamp bo nie do konca znormalizowane te wektory sa */
	mediump float fres2 = fres * fres; /* szybciej tak niz potega */
	mediump float fresnel = 1.0 + FRESNEL * fres2 * fres2 * fres;
	enviroment *= fresnel;	
	
	gl_FragColor = vec4(color * MATERIAL_DIFFUSE.rgb + materialDiffuseAndSpec.a * enviroment * MATERIAL_ENVIRONMENT, MATERIAL_DIFFUSE.a);
	
	/*gl_FragColor = vec4(enviroment, 1.0);*/
}
