uniform mediump float SHININESS;
uniform mediump float FRESNEL;

uniform mediump mat3 EVIRONMENT_TEXTURE_MATRIX;

uniform lowp vec3 LIGHTANDMATERIAL_AMBIENT;

uniform lowp vec4 MATERIAL_DIFFUSE;
uniform lowp vec3 MATERIAL_SPECULAR;
uniform lowp vec3 MATERIAL_EMISSIVE;
uniform lowp vec3 MATERIAL_ENVIRONMENT;

uniform lowp vec3 LIGHT0_LIGHTDIR;
uniform lowp vec3 LIGHT0_LIGHTCOLOR;
uniform lowp vec3 LIGHT1_LIGHTCOLOR;
uniform mediump float LIGHT1_ATTENUATION;

/* Diffuse and alpha/specular */
uniform lowp sampler2D TEXTURE0;

/* Normal map (RGB), illumination (A) */
uniform lowp sampler2D TEXTURE1;

/* enviroment map */
uniform lowp samplerCube TEXTURE2;

varying mediump vec3 Light1DirAndDist;
varying lowp vec3 Normal;
varying lowp vec3 Tangent;
varying lowp vec3 Bitangent;
varying mediump vec2 TexCoord;
varying mediump vec3 ViewDirAndDist;
varying lowp vec3 Light0_HalfDir;
varying lowp vec3 Light1_HalfDir;

void main()
{	
	mediump vec3 ViewDir = normalize(ViewDirAndDist);

	lowp vec4 materialDiffuseAndSpec = texture2D(TEXTURE0, TexCoord);
	
	lowp vec4 tex1 = texture2D(TEXTURE1, TexCoord);
	lowp vec3 normalMap = vec3(tex1.r * 2.0 - 1.0, tex1.g * 2.0 - 1.0, tex1.b);
	lowp float illum = tex1.a;
	
	lowp vec3 normal = normalMap.x * Tangent + normalMap.y * Bitangent + normalMap.z * Normal;	
	
	mediump vec3 envDir = -reflect(ViewDir, normal); /* minus bo viewDir to eye-vertex, a reflect to ViewDir - 2*dot(ViewDir, normal) * normal */
	mediump vec3 envTexCoord = EVIRONMENT_TEXTURE_MATRIX * envDir;	
	lowp vec3 enviroment = textureCube(TEXTURE2, envTexCoord).rgb;	

	lowp vec3 Light1Dir = normalize(Light1DirAndDist);
	lowp float att = 1.0 / (1.0 + LIGHT1_ATTENUATION * dot(Light1DirAndDist, Light1DirAndDist));	
	
	lowp vec3 diffuseLight = LIGHT0_LIGHTCOLOR * max(dot(normal, LIGHT0_LIGHTDIR), 0.0) +
							 LIGHT1_LIGHTCOLOR * max(dot(normal, Light1Dir), 0.0) * att;
					  
	lowp vec3 specularLight = LIGHT0_LIGHTCOLOR * pow(max(dot(Light0_HalfDir, normal), 0.0), SHININESS) +
							  LIGHT1_LIGHTCOLOR * pow(max(dot(Light1_HalfDir, normal), 0.0), SHININESS) * att;

	mediump float fres = 1.0 - clamp(dot(normal, ViewDir), 0.0, 1.0); /* musi byc clamp bo nie do konca znormalizowane te wektory sa */
	mediump float fres2 = fres * fres; /* szybciej tak niz potega */
	mediump float fresnel = 1.0 + FRESNEL * fres2 * fres2 * fres;
	specularLight *= fresnel;
	enviroment *= fresnel;							  
							  
	gl_FragColor = vec4(materialDiffuseAndSpec.rgb * (diffuseLight * MATERIAL_DIFFUSE.rgb + LIGHTANDMATERIAL_AMBIENT + vec3(illum, illum, illum) * MATERIAL_EMISSIVE) +				
						materialDiffuseAndSpec.a * (specularLight * MATERIAL_SPECULAR + enviroment * MATERIAL_ENVIRONMENT),
						MATERIAL_DIFFUSE.a);
}
