uniform mediump float SHININESS;
uniform mediump float FRESNEL;
uniform mediump float PARALAX_STRENGHT;

uniform lowp vec3 LIGHTANDMATERIAL_AMBIENT;

uniform lowp vec4 MATERIAL_DIFFUSE;
uniform lowp vec3 MATERIAL_SPECULAR;
uniform lowp vec3 MATERIAL_EMISSIVE;

uniform lowp vec3 LIGHT0_LIGHTDIR;
uniform lowp vec3 LIGHT0_LIGHTCOLOR;

/* Diffuse and alpha/specular */
uniform lowp sampler2D TEXTURE0;

/* Normal map (RG), bump map (B), illumination (A) */
uniform lowp sampler2D TEXTURE1;

varying lowp vec3 Normal;
varying lowp vec3 Tangent;
varying lowp vec3 Bitangent;
varying mediump vec2 TexCoord;
varying mediump vec3 ViewDirAndDist;
varying lowp vec3 Light0_HalfDir;

void main()
{	
	mediump vec3 ViewDir = normalize(ViewDirAndDist);

	mediump vec2 offset;
	offset.x = dot(ViewDir, Tangent);
	offset.y = dot(ViewDir, Bitangent);
	mediump vec2 TexCoordParalaxed = TexCoord + offset * ((texture2D(TEXTURE1, TexCoord).b * 2.0 - 1.0) * PARALAX_STRENGHT);
	
	lowp vec4 materialDiffuseAndSpec = texture2D(TEXTURE0, TexCoordParalaxed);
	
	lowp vec4 tex1 = texture2D(TEXTURE1, TexCoordParalaxed);
	tex1.xy = tex1.xy * 2.0 - 1.0;
	lowp vec3 normalMap = vec3(tex1.x, tex1.y, sqrt(1.0 - tex1.x * tex1.x - tex1.y * tex1.y));
	lowp float illum = tex1.a;
	
	lowp vec3 normal = normalMap.x * Tangent + normalMap.y * Bitangent + normalMap.z * Normal;
	
	lowp vec3 diffuseLight = LIGHT0_LIGHTCOLOR * max(dot(normal, LIGHT0_LIGHTDIR), 0.0);
	lowp vec3 specularLight = LIGHT0_LIGHTCOLOR * pow(max(dot(normal, Light0_HalfDir), 0.0), SHININESS);

	mediump float fres = 1.0 - clamp(dot(normal, ViewDir), 0.0, 1.0); /* musi byc clamp bo nie do konca znormalizowane te wektory sa */
	mediump float fres2 = fres * fres; /* szybciej tak niz potega */
	mediump float fresnel = 1.0 + FRESNEL * fres2 * fres2 * fres;
	specularLight *= fresnel;
	
	gl_FragColor = vec4(materialDiffuseAndSpec.rgb * (diffuseLight * MATERIAL_DIFFUSE.rgb + LIGHTANDMATERIAL_AMBIENT + vec3(illum, illum, illum) * MATERIAL_EMISSIVE) +				
						materialDiffuseAndSpec.a * specularLight * MATERIAL_SPECULAR,
						MATERIAL_DIFFUSE.a);
}
