uniform mediump float SHININESS;
uniform mediump float FRESNEL;
uniform mediump float PARALAX_STRENGHT;

uniform lowp vec3 LIGHTANDMATERIAL_AMBIENT;

uniform lowp vec4 MATERIAL_DIFFUSE;
uniform lowp vec3 MATERIAL_SPECULAR;

uniform lowp vec3 LIGHT0_LIGHTDIR;
uniform lowp vec3 LIGHT0_LIGHTCOLOR;
uniform lowp vec3 LIGHT1_LIGHTCOLOR;
uniform mediump float LIGHT1_ATTENUATION;

uniform highp float TIME;
uniform mediump float PARAM1;
uniform mediump float PARAM2;

/* Diffuse and alpha/specular */
uniform lowp sampler2D TEXTURE0;

/* Refraction map IN warp*/
uniform lowp sampler2D TEXTURE1;

/* Refraction map BEFORE warp*/
uniform lowp sampler2D TEXTURE2;

varying mediump vec3 Light1DirAndDist;
varying lowp vec3 Normal;
varying lowp vec3 Tangent;
varying lowp vec3 Bitangent;
varying mediump vec2 TexCoord;
varying lowp vec3 Light0_HalfDir;
varying lowp vec3 Light1_HalfDir;
varying mediump vec2 DeferredTexCoord;

void main()
{
	mediump float A = 8.0 * PARAM2;
	mediump float B = 2.0;
	mediump float C = 0.3 * PARAM2;
	mediump float hC = C * 0.5;
	
	mediump vec2 t = (TexCoord - vec2(0.5, 0.5)) * 2.0;
	mediump float k = dot(t, t) * A + TIME * B;
	
	/*mediump float fx = sin(k) * C;	
	mediump vec2 TexCoord2 = t * (fx * 0.5) + vec2(0.5, 0.5); - optymalizacja nizej*/
	
	mediump vec2 fx = t * (sin(k) * hC);
	
	mediump vec2 TexCoord2 = fx + vec2(0.5, 0.5);
	mediump vec2 DeferredTexCoord2 = DeferredTexCoord + fx;
	
	mediump vec3 tsNormal = vec3(cos(k) * 2.0 * A * t.x, cos(k) * 2.0 * A * t.y, -1.0);
	mediump vec3 normal = tsNormal.x * Tangent + tsNormal.y * Bitangent + tsNormal.z * Normal;
	normal = normalize(normal);	
	
	mediump float alpha = clamp(1.0 + max(normal.z * 5.0, 0.0) - PARAM1 * 6.0, 0.0, 1.0);
		
	lowp vec4 materialDiffuseAndSpec = texture2D(TEXTURE0, TexCoord2);
	lowp vec3 transparency1 = texture2D(TEXTURE2, DeferredTexCoord2).rgb;	/* tu 3 bo to refraction, a 2 to nowy wymiar */
	lowp vec3 transparency2 = texture2D(TEXTURE1, DeferredTexCoord2).rgb;	
	lowp vec3 transparency = transparency1 * alpha + transparency2 * (1.0 - alpha);  /* dobrze jest, bo alpha to jakby MINUS PARAM1 */
	
	lowp vec3 Light1Dir = normalize(Light1DirAndDist);
	lowp float att = 1.0 / (1.0 + LIGHT1_ATTENUATION * dot(Light1DirAndDist, Light1DirAndDist));
	
	lowp vec3 diffuseLight = LIGHT0_LIGHTCOLOR * max(dot(normal, LIGHT0_LIGHTDIR), 0.0)
						   + LIGHT1_LIGHTCOLOR * max(dot(normal, Light1Dir), 0.0) * att;
					  
	lowp vec3 specularLight = LIGHT0_LIGHTCOLOR * pow(max(dot(normal, Light0_HalfDir), 0.0), SHININESS)
							+ LIGHT1_LIGHTCOLOR * pow(max(dot(normal, Light1_HalfDir), 0.0), SHININESS) * att;

	mediump float fres = 1.0 - clamp(normal.z, 0.0, 1.0); /* musi byc clamp bo nie do konca znormalizowane te wektory sa */
	mediump float fres2 = fres * fres; /* szybciej tak niz potega */
	mediump float fresnel = 1.0 + FRESNEL * fres2 * fres2 * fres;
	specularLight *= fresnel;					   
					   
	gl_FragColor = vec4(materialDiffuseAndSpec.rgb * (diffuseLight * MATERIAL_DIFFUSE.rgb + LIGHTANDMATERIAL_AMBIENT) +				
						materialDiffuseAndSpec.a * (specularLight * MATERIAL_SPECULAR) +
						transparency,
					    MATERIAL_DIFFUSE.a);
}
