uniform mediump float SHININESS;
uniform mediump float FRESNEL;
uniform mediump float PARALAX_STRENGHT;

uniform lowp vec3 LIGHTANDMATERIAL_AMBIENT;

uniform lowp vec4 MATERIAL_DIFFUSE;
uniform lowp vec3 MATERIAL_SPECULAR;
uniform lowp vec3 MATERIAL_EMISSIVE;

uniform lowp vec3 LIGHT0_LIGHTDIR;
uniform lowp vec3 LIGHT0_LIGHTCOLOR;
uniform lowp vec3 LIGHT1_LIGHTCOLOR;
uniform mediump float LIGHT1_ATTENUATION;

uniform mediump float FOG_DISTANCE_FROM;
uniform mediump float FOG_DISTANCE_TO;

/* Diffuse and alpha */
uniform lowp sampler2D TEXTURE0;

/* Normal map and bump map */
uniform lowp sampler2D TEXTURE1;

/* Illumination (emission) and specular map */
uniform lowp sampler2D TEXTURE2;

varying mediump vec3 Light1DirAndDist;
varying lowp vec3 Normal;
varying lowp vec3 Tangent;
varying lowp vec3 Bitangent;
varying mediump vec2 TexCoord;
varying mediump vec3 ViewDirAndDist;
varying lowp vec3 Light0_HalfDir;
varying lowp vec3 Light1_HalfDir;
varying mediump float Distance;

void main()
{
	mediump vec3 ViewDir = normalize(ViewDirAndDist);

	lowp float alpha = (FOG_DISTANCE_TO - Distance) / (FOG_DISTANCE_TO - FOG_DISTANCE_FROM);

	mediump vec2 offset;
	offset.x = dot(ViewDir, Tangent);
	offset.y = dot(ViewDir, Bitangent);
	mediump vec2 TexCoordParalaxed = TexCoord + offset * ((texture2D(TEXTURE1, TexCoord).a * 2.0 - 1.0) * PARALAX_STRENGHT);	
	
	lowp vec4 materialColor = texture2D(TEXTURE0, TexCoordParalaxed);
	lowp vec3 normalMap = texture2D(TEXTURE1, TexCoordParalaxed).rgb * 2.0 - 1.0;
	lowp vec4 illumAndSpec  = texture2D(TEXTURE2, TexCoordParalaxed);	
	
	lowp vec3 normal = normalMap.x * Tangent + normalMap.y * Bitangent + normalMap.z * Normal;	
	
	lowp vec3 Light1Dir = normalize(Light1DirAndDist);
	lowp float att = 1.0 / (1.0 + LIGHT1_ATTENUATION * dot(Light1DirAndDist, Light1DirAndDist));	

	lowp vec3 diffuseLight = LIGHT0_LIGHTCOLOR * max(dot(normal, LIGHT0_LIGHTDIR), 0.0) +
							 LIGHT1_LIGHTCOLOR * max(dot(normal, Light1Dir), 0.0) * att;
					  
	lowp vec3 specularLight = LIGHT0_LIGHTCOLOR * pow(max(dot(normal, Light0_HalfDir), 0.0), SHININESS) +
							  LIGHT1_LIGHTCOLOR * pow(max(dot(normal, Light1_HalfDir), 0.0), SHININESS) * att;

	mediump float fres = 1.0 - clamp(dot(normal, ViewDir), 0.0, 1.0); /* musi byc clamp bo nie do konca znormalizowane te wektory sa */
	mediump float fres2 = fres * fres; /* szybciej tak niz potega */
	mediump float fresnel = 1.0 + FRESNEL * fres2 * fres2 * fres;
	specularLight *= fresnel;						  
							 							  
	gl_FragColor = vec4(materialColor.rgb * (diffuseLight * MATERIAL_DIFFUSE.rgb + LIGHTANDMATERIAL_AMBIENT + illumAndSpec.rgb * MATERIAL_EMISSIVE) +				
						illumAndSpec.a * specularLight * MATERIAL_SPECULAR,
						materialColor.a * MATERIAL_DIFFUSE.a * alpha);
}
