uniform mediump mat4 PROJECTIONMATRIX;
uniform mediump mat4 MODELVIEWMATRIX;

uniform lowp vec3 LIGHT0_LIGHTDIR;
uniform lowp vec3 LIGHT0_LIGHTCOLOR;
uniform mediump vec3 LIGHT1_LIGHTPOS;
uniform lowp vec3 LIGHT1_LIGHTCOLOR;
uniform mediump float LIGHT1_ATTENUATION;

uniform lowp vec4 MATERIAL_DIFFUSE;

uniform lowp vec3 LIGHTANDMATERIAL_AMBIENT;

attribute highp vec3 inVertex;
attribute lowp vec3 inNormal;
attribute mediump vec2 inTexCoord;

varying lowp vec4 Color;
varying mediump vec2 TexCoord;

void main(void)
{
	mediump mat3 NormalMatrix = mat3(MODELVIEWMATRIX[0].xyz, MODELVIEWMATRIX[1].xyz, MODELVIEWMATRIX[2].xyz);

	gl_Position = PROJECTIONMATRIX * (MODELVIEWMATRIX * vec4(inVertex, 1.0));
	TexCoord = inTexCoord;
	
	mediump vec3 vertex = (MODELVIEWMATRIX * vec4(inVertex, 1.0)).xyz;
	mediump vec3 normal = NormalMatrix * inNormal;
	mediump vec3 light1Dir = LIGHT1_LIGHTPOS - vertex;
	mediump float sqrdist = dot(light1Dir, light1Dir);
	light1Dir = normalize(light1Dir);
	
	Color = vec4((LIGHT0_LIGHTCOLOR * max(dot(normal, LIGHT0_LIGHTDIR), 0.0)
			   + LIGHT1_LIGHTCOLOR * max(dot(normal, light1Dir), 0.0) / (1.0 + LIGHT1_ATTENUATION * sqrdist)) * MATERIAL_DIFFUSE.rgb
			   + LIGHTANDMATERIAL_AMBIENT
			   , MATERIAL_DIFFUSE.a);
}
